#
# provides access to /etc/group
#
# Created: 1998-02-04 by Berend
#

# Copyright (c) 1998
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

package access_group;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_all_local_groups get_all_local_groups_with_member_info get_local_group_name get_local_gid get_local_group_properties change_local_group get_new_local_gid remove_local_group);


#
# read every group in /etc/group
# Parameters: 
#   input <none>
#   output @groups
#
sub get_all_local_groups {

    my @groups = ();
    my @fields = ();
    my $i = 0;

    open (FILE, '/usr/sbin/pw showgroup -a |');

    while ($Line = <FILE>) {
	@fields = split /:/, $Line, 2;
	@groups[$i++] = $fields[0];
    }

    close (FILE);

    return sort @groups;

}


#
# read every group in /etc/group, include information of the given
# user is a member of that group
# Parameters: 
#   input $user
#   output %groups
#
sub get_all_local_groups_with_member_info {

    my($user) = @_;
    my %groups = ();
    my @fields = ();

    open (FILE, '/usr/sbin/pw showgroup -a |');

    while ($Line = <FILE>) {
	@fields = split /:/, $Line;
	if ( ($user ne '') &&
	     ($fields[3] =~ /$user/) ) {
	    $groups{$fields[0]} = 'YES';
	}
	else {
	    $groups{$fields[0]} = 'NO';
	}
    }

    close (FILE);

    return %groups;

}


#
# return name for given gid
# Parameters: 
#   input $gid
#   output $groupname
#
sub get_local_group_name {

    my($gid) = @_;
    my $groupname = ();
    my @fields = ();

    open (FILE, "/usr/sbin/pw showgroup -g $gid |");

    while ($Line = <FILE>) {
	@fields = split /:/, $Line, 2;
	$groupname = $fields[0];
	last;
    }

    close (FILE);

    return $groupname;

}


#
# return gid for given group name
# Parameters: 
#   input $groupname
#   output $gid
#
sub get_local_gid {

    my($groupname) = @_;
    my($gid) = 0;
    my @fields = ();

    open (FILE, "- | ") or exec '/usr/sbin/pw', 'showgroup', '-n', "$groupname";

    while ($Line = <FILE>) {
	@fields = split /:/, $Line;
	$gid = $fields[2];
	last;
    }

    close (FILE);

    return $gid;

}


#
# read properties of a given group
# Parameters: 
#   input $group
#   output %group_properties
#
sub get_local_group_properties {

    my($group) = @_;
    my @group_properties = ();

    open (FILE, "- |") or exec '/usr/sbin/pw', 'showgroup', '-n', "$group";

    while ($Line = <FILE>) {
	chop($Line);
	@fields = split /:/, $Line;
	$group_properties{'name'} = $fields[0];
	$group_properties{'gid'} = $fields[2];
	$group_properties{'members'} = $fields[3];
    }

    close (FILE);

    return %group_properties;

}


#
# change local group
# input: $group, $gid
# output: $modified, $errormsg, %oldproperties
#

sub change_local_group {

    my($group) = @_;
    shift;
    my($gid) = @_;
    shift;
    my(%properties) = @_;
    my %oldproperties = ();
    my $modified = 0;
    my $newgroup = ($group eq '');
    my $errormsg = '';
    my $user;

    if ($newgroup == 0) {
	%oldproperties = get_local_group_properties($group);
    }

    if ( ($properties{'name'} ne $oldproperties{'name'}) ||
	 ($properties{'gid'} ne $oldproperties{'gid'}) ||
	 ($properties{'members'} ne $oldproperties{'members'}) ) {

	$modified = 1;

	# group renamed?
	if ( ($group ne '') &&
	     ($group ne $properties{'name'}) ) {
	    $rc = 0xffff & system "/usr/sbin/pw", 'groupmod', 
	    '-g', "$gid",
	    '-l', "$properties{'name'}";
	    if ($rc != 0) { $errormsg = 'group rename failed'; }
	}

	# new or existing user?
	@args = ('-n', "$properties{'name'}", '-g', "$properties{'gid'}");
	push @args, '-M', "$properties{'members'}";
	if ($newgroup != 0) {
	    $rc = 0xffff & system "/usr/sbin/pw", 'groupadd', @args;
	    if ($rc != 0) { $errormsg = 'Group addition failed'; }
	}
	else {
	    $rc = 0xffff & system "/usr/sbin/pw", 'groupmod', @args;
	    if ($rc != 0) { $errormsg = 'Group modification failed'; }
	}

    }

    return ($modified, $errormsg, %oldproperties);

}


#
# return highest gid + 1 in /etc/group
# Parameters: 
#   input <none>
#   output $gid
#
sub get_new_local_gid {

    my $gid = 1000;
    my @fields = ();

    open (FILE, '/usr/sbin/pw showgroup -a |');

    while ($Line = <FILE>) {
	@fields = split /:/, $Line;
	if ( ($fields[2] < 65500) &&
	     ($fields[2] > $gid) ) {
	    $gid = $fields[2] + 1;
	}
    }

    close (FILE);

    return $gid;

}


#
# remove a local group
# input: $group
#
sub remove_local_group {

    my($group) = @_;
    $rc = 0xffff & system '/usr/sbin/pw', 'groupdel', '-n', "$group";

}
