#
# provides access to /etc/passwd
#
# Created: 1998-02-03 by Berend
#

# Copyright (c) 1998
#      Brothers de Boer.  All rights reserved.
#  
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer,
#    verbatim and that no modifications are made prior to this
#    point in the file.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY Brothers de Boer ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL WOUTER DE BOER OR BEREND DE BOER OR ANY
# OTHER OF THEIR BROTHERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, LIFE 
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

package access_passwd;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_all_local_users get_local_user_properties get_new_local_uid change_local_user remove_local_user);

use POSIX;
use access_group;

#
# read every user in /etc/passwd
# Parameters: 
#   input <none>
#   output @users
#
sub get_all_local_users {

    my @users = ();
    my @fields = ();
    my $i = 0;

    open (FILE, '/usr/sbin/pw showuser -a |');

    while ($Line = <FILE>) {
	@fields = split /:/, $Line, 2;
	@users[$i++] = $fields[0];
    }

    close (FILE);

    return sort @users;

}



#
# read properties of a given user
# Parameters: 
#   input $username
#   output %user_properties
#
sub get_local_user_properties {

    my($user) = @_;
    my @user_properties = ();

    open (FILE, "- |") or exec '/usr/sbin/pw', 'showuser', '-n', "$user";

    while ($Line = <FILE>) {
	chop($Line);
	@fields = split /:/, $Line;
	$user_properties{'name'} = $fields[0];
	$user_properties{'uid'} = $fields[2];
	$user_properties{'gid'} = $fields[3];
	$user_properties{'class'} = $fields[4];
	if ($fields[5] == 0) {
	    $user_properties{'change'} = 'never';
	}
	else {
	    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
		localtime($fields[5]);
	    $user_properties{'change'} = strftime("%d-%b-%Y %X", $sec, $min, $hour, $mday, $mon, $year);
	}
	if ($fields[6] == 0) {
	    $user_properties{'expire'} = 'never';
	}
	else {
	    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
		localtime($fields[6]);
	    $user_properties{'expire'} = strftime("%d-%b-%Y %X", $sec, $min, $hour, $mday, $mon, $year);
	}
	$user_properties{'gecos'} = $fields[7];
	$user_properties{'homedir'} = $fields[8];
	if ($fields[9] eq '')
	{
	    $user_properties{'shell'} = '/bin/sh';
	}
	else {
	    $user_properties{'shell'} = $fields[9];
	}
	@fields = split /,/, $user_properties{'gecos'};
	$user_properties{'fullname'} = $fields[0];
	$user_properties{'office'} = $fields[1];
	$user_properties{'wphone'} = $fields[2];
	$user_properties{'hphone'} = $fields[3];
    }

    close (FILE);

    return %user_properties;

}


#
# return highest uid + 1 in /etc/passwd
# Parameters: 
#   input <none>
#   output $uid
#
sub get_new_local_uid {

    my $uid = 1000;
    my @fields = ();

    open (FILE, '/usr/sbin/pw showuser -a |');

    while ($Line = <FILE>) {
	@fields = split /:/, $Line;
	if ( ($fields[2] < 65500) &&
	     ($fields[2] > $uid) ) {
	    $uid = $fields[2] + 1;
	}
    }

    close (FILE);

    return $uid;

}


#
# modify properties of a given user or add user if username is empty
# input: $user, $uid, %userproperties
# PRE: %userproperties contains valid name and uid
# output: $modified, %oldproperties

sub change_local_user {

    my($user) = @_;
    shift;
    my($uid) = @_;
    shift;
    my(%properties) = @_;
    my %oldproperties = ();
    my $modified = 0;
    my $newuser = ($user eq '');
    my $errormsg = '';
    my %oldgroups = ();
    my $oldgrouplist = '';
    my $group;

    if ($newuser == 0) {
	%oldproperties = get_local_user_properties($user);
    }
    
    #construct gecos field
    if ( ($properties{'office'} eq '') &&
	 ($properties{'wphone'} eq '') &&
	 ($properties{'hpone'} eq '') ) {
	$properties{'gecos'} = $properties{'fullname'};
    }
    else {
	$properties{'gecos'} = "$properties{'fullname'},$properties{'office'},$properties{'wphone'},$properties{'hphone'}";
    }
    # create old grouplist
    if ($newuser == 0) {
	(%oldgroups) = get_all_local_groups_with_member_info($user);
	$oldgrouplist = '';
	foreach $group (keys %oldgroups) {
	    if ($oldgroups{$group} eq 'YES') {
		if ($oldgrouplist eq '') { $oldgrouplist = $group; }
		else { $oldgrouplist = "$oldgrouplist,$group"; }
	    }
	}
	$oldproperties{'grouplist'} = $oldgrouplist;
    }
   
    # if anything changed call pw
    if ( ($properties{'name'} ne $oldproperties{'name'}) ||
	 ($properties{'gecos'} ne $oldproperties{'gecos'}) ||
	 ($properties{'uid'} ne $oldproperties{'uid'}) ||
	 ($properties{'gid'} ne $oldproperties{'gid'}) ||
	 ($properties{'homedir'} ne $oldproperties{'homedir'}) ||
	 ($properties{'shell'} ne $oldproperties{'shell'}) ||
	 ($properties{'shell'} ne $oldproperties{'shell'}) ||
	 ($properties{'class'} ne $oldproperties{'class'}) ||
	 ($properties{'change'} ne $oldproperties{'change'}) ||
	 ($properties{'expire'} ne $oldproperties{'expire'}) ||
	 ($properties{'grouplist'} ne $oldproperties{'grouplist'}) ) {

	$modified = 1;

	# change change/expire fields from 'never' to 0
	if ( ($properties{'change'} eq '') ||
	     ($properties{'change'} =~ /^never/i) ) {
	    $properties{'change'} = 0;
	}
	if ( ($properties{'expire'} eq '') ||
	     ($properties{'expire'} =~ /^never/i) ) {
	    $properties{'expire'} = 0;
	}

	# user renamed? Remove it, next add it
	if ( ($user ne '') &&
	     ($user ne $properties{'name'}) ) {
	    $rc = 0xffff & system "/usr/sbin/pw", 'userdel', 
	    '-n', "$user", 
	    '-u', "$uid";
	    $user = '';
	    if ($rc != 0) { $errormsg = 'User rename failed'; }
	}

	# new or existing user?
	if ($newuser != 0) {
	    @args = ('-m','-o','-h','-');	    
	    if ($properties{'grouplist'} ne '') {
		push @args, '-G', "$properties{'grouplist'}";
	    }
	    if ($properties{'expire'} != 0) { 
		push @args, '-e', "$properties{'expire'}";
	    }
	    if ($properties{'change'} != 0) { 
		push @args, '-p', "$properties{'change'}";
	    }
	    $rc = 0xffff & system '/usr/sbin/pw', 'useradd',
	    '-n', "$properties{'name'}", 
	    '-u', "$properties{'uid'}",
	    '-g', "$properties{'gid'}",
	    '-c', "$properties{'gecos'}",
	    '-d', "$properties{'homedir'}",
	    '-s', "$properties{'shell'}",
	    @args;
	    if ($rc != 0) { $errormsg = 'User add failed'; }
	}
	else {
	    @args = ('-m');
	    if ($properties{'grouplist'} ne '') {
		push @args, '-G', "$properties{'grouplist'}";
	    }
	    $rc = 0xffff & system '/usr/sbin/pw', 'usermod',
	    '-n', "$user", 
	    '-u', "$properties{'uid'}",
	    '-g', "$properties{'gid'}",
	    '-c', "$properties{'gecos'}",
	    '-d', "$properties{'homedir'}",
	    '-s', "$properties{'shell'}",
	    '-e', "$properties{'expire'}",
	    '-p', "$properties{'change'}",
	    @args;
	    if ($rc != 0) { $errormsg = 'User modification failed'; }
	}
    }

    #set password
    if ( ($errormsg eq '') &&
	 ($properties{'plainpw'} ne '') ) {
	$modified = 1;
	$rc = 0xffff & system '/bin/sh', '-c', 
	"echo '$properties{'plainpw'}' | /usr/sbin/pw usermod -n '$properties{'name'}' -u '$properties{'uid'}' -h 0";
	if ($rc != 0) { $errormsg = 'Password change failed'; }
    }

    return ($modified, $errormsg, %oldproperties);

}


#
# remove a local user
# input: $user
#
sub remove_local_user {

    my($user) = @_;
    shift;
    my($removehomedir) = @_;

    if ($removehomedir =~ /^yes$/i) {
	$rc = 0xffff & system '/usr/sbin/pw', 'userdel', '-n', "$user", '-r';
    }
    else {
	$rc = 0xffff & system '/usr/sbin/pw', 'userdel', '-n', "$user";
    }

}
